




/*********Circuit Connections*************
      Digital Pot  |   Arduino           | 
          CS       |     D10 (SS)        |
          SI       |     D11 (MOSI)      |
          CLK      |     D13 (SCK)       |
      VCC,PA0,PA1  |       VCC           |
      GND,PB0,PB1  |       GND           |
          W0       |                     |
          W1       |                     |
 *****************************************/

/***********************Notice********************************
   1.Resistor terminals A, B and W have no restrictions on
     polarity with respect to each other.
   2.Current through terminals A, B and W should not excceed ±1mA.
   3.Voltages on terminals A, B and W should be within 0 - VCC.
   4.Resistance value byte (0 - 255)
**************************************************************/

#include <SPI.h>

// set pin 10 as the slave select (SS) for the digital pot
// when using Arduino UNO/nano

const int CS_PIN = 10;         // set pin 10 as the slave select (SS) for the digital Potenziometer MCP42100
const int detectIn = 12;       // input to detect if an INSTRUMENT INPUT Jack has been inserted
const int Stop = A0;           // input for STOP switch
const int Fade = A1;           // input for FADE+TREMOLO+UNDO/REDO switch
const int potPinFade = A2;     // input for FADE/TREMOLO INTENSITY Potentiometer
const int potPinTremolo = A3;  // input for TREMOLO SPEED potenziometer
const int stopTip = A4;        // input for TIP JACK -> STOP command to Loop NC
const int stopRing = A5;       // input for RING JACK -> UNDO/REDO command to Loop NC

int inState = 0;
int fadeState = 0;
int timeFade = 100;
int timeFadeUp = 10;
int speedTremolo = 100;
int stopState = 0;
int potValue = 255;             // value of the digital Potenziometer MCP42100 (255 --> MAX -> no volume attenuation)
int Count = 0;
int stopCount = 0;
int inter = 0;
int intensity = 0;             // intensity volume TREMOLO value from 0 to 100
int volume = 255;              // modulated volume TREMOLO

float pressLength_milliSeconds = 0;  // to define how long the Foot Switch has been pressed
int optionOne_milliSeconds = 100;    // short press -> ON/OFF function
int optionTwo_milliSeconds = 450;    // long press  -> Leslie Speed-up
int onOff = 0;                       // flag if B9 is OFF -> 0     ON -> 1 
int latchSim = 0;                    // flag to simulate latching of the foot switch
int inc = 1;                         // increment volume +/-1 for TREMOLO


/***********************MCP42XXX Commands************************/
//potentiometer select byte
const int POT0_SEL = 0x11;            // command to the digital potenziometer 0 pin(8-9-10)
const int POT1_SEL = 0x12;            // command to the digital potenziometer 1 pin(5-6-7)
const int BOTH_POT_SEL = 0x13;        // command to the digital potenziometers 0+1 pin(5-6-7)+(8-9-10)

//shutdown the device to put it into power-saving mode.
//In this mode, terminal A is open-circuited and the B and W terminals are shorted together.
//send new command and value to exit shutdowm mode.
//const int POT0_SHUTDOWN = 0x21;
//const int POT1_SHUTDOWN = 0x22;
//const int BOTH_POT_SHUTDOWN = 0x23;

/*****************************************************************/


#include "SevSeg.h"
SevSeg sevseg;                // initiate the seven segment display

#define MAX_NUMBER_STRINGS 20
#define MAX_STRING_SIZE 2
char testStrings[MAX_NUMBER_STRINGS][MAX_STRING_SIZE];

#define PATTERN_CHANGE_TIME 500
unsigned long timer = millis() - PATTERN_CHANGE_TIME;

byte testStringsPos = 19;




void setup() {

  pinMode(CS_PIN, OUTPUT);           // set the CS_PIN as an output:
  SPI.begin();                       // initialize SPI:
  pinMode(Fade, INPUT_PULLUP);       // Fade INPUT -> switch pressed = LOW
  pinMode(Stop, INPUT_PULLUP);       // Stop INPUT -> switch pressed = LOW
  pinMode(detectIn, INPUT_PULLUP);   // Jack INPUT -> if inserted = LOW
  pinMode(stopTip, OUTPUT);          // Stop reed relay
  pinMode(stopRing, OUTPUT);         // Undo/Redo reed relay
  digitalWrite(stopTip, HIGH);       // Stop reed relay -> it has to be NC (Normally Closed) - BOSS Standard for Looper comman
  digitalWrite(stopRing, HIGH);      // Undo/Redo reed relay -> it has to be NC (Normally Closed) - BOSS Standard for Looper command
  
  byte numDigits = 1;
  byte digitPins[] = {};  //leave empty for a single digit display
  byte segmentPins[] = {2, 3, 4, 5, 6, 7, 8, 9}; //input on Arduino -> A, B , C, D, E, F, G, DP on the display
  bool resistorsOnSegments = true; // 'false' means resistors are on digit pins
  byte hardwareConfig = COMMON_ANODE;
  bool updateWithDelays = false; // Default. Recommended
  bool leadingZeros = false; // Use 'true' if you'd like to keep the leading zeros

  sevseg.begin(hardwareConfig, numDigits, digitPins, segmentPins, resistorsOnSegments, updateWithDelays, leadingZeros);
  sevseg.setBrightness(90);

  // set of test strings for 7 segment display
  
  strcpy(testStrings[0], "0");
  strcpy(testStrings[1], "1");
  strcpy(testStrings[2], "2");
  strcpy(testStrings[3], "3");
  strcpy(testStrings[4], "4");
  strcpy(testStrings[5], "5");
  strcpy(testStrings[6], "6");
  strcpy(testStrings[7], "7");
  strcpy(testStrings[8], "8");
  strcpy(testStrings[9], "9");
  strcpy(testStrings[10], "0.");
  strcpy(testStrings[11], "F");
  strcpy(testStrings[12], "A");
  strcpy(testStrings[13], "D");
  strcpy(testStrings[14], "E");
  strcpy(testStrings[15], "S");
  strcpy(testStrings[16], "T");
  strcpy(testStrings[17], "U");
  strcpy(testStrings[18], "R");
  strcpy(testStrings[19], ".");

             }



// Function Declaration:  send the command to 7 segments display ********************************

void display7(){
       sevseg.setChars(testStrings[testStringsPos]);
       sevseg.refreshDisplay();
              }
//  *********************************************************************************************


// Function Declaration:  send the command and the wiper value to the digital potenziometer through SPI **********************

void DigitalPotWrite(int cmd, int val)
{
  // constrain input value within 0 - 255
  val = constrain(val, 0, 255);
  // set the CS pin to low to select the chip:
  digitalWrite(CS_PIN, LOW);
  // send the command and value via SPI:
  SPI.transfer(cmd);
  SPI.transfer(val);
  // Set the CS pin high to execute the command:
  digitalWrite(CS_PIN, HIGH);
}
 
//  **********************************************************************************************





void loop() {

   inState = digitalRead(detectIn);
   delay(25);
   fadeState = digitalRead(Fade);
   delay(25);
   stopState = digitalRead(Stop);
   delay(25);
   timeFade = analogRead(potPinFade)/25;
   timeFadeUp = timeFade/2;
   speedTremolo = analogRead(potPinTremolo);
   display7(); 
   delay(100); 
  
   
   DigitalPotWrite(BOTH_POT_SEL , potValue);   // value of the digital Potenziometer MCP42100 (255 --> MAX -> no volume attenuation)
                       

if(inState == HIGH){                                                  // if NO Jack Instrument INPUT is detected -> act as STOP/UNDO/REDO switches
             
             while(digitalRead(Stop) == LOW){digitalWrite(stopTip, LOW); 
             testStringsPos = 15;display7(); delay(250);
             digitalWrite(stopTip, HIGH); }

            if(digitalRead(Fade) == LOW){
  
                    switch (Count) {
                           case 0:       // UNDO
                            digitalWrite(stopRing, LOW); 
                            testStringsPos = 18;display7(); delay(250);
                            digitalWrite(stopRing, HIGH);
                            Count++;
                           break; 
                           
                             case 1:       // REDO
                               digitalWrite(stopRing, LOW); 
                               testStringsPos = 17;display7(); delay(250);
                               digitalWrite(stopRing, HIGH);
                               Count = 0;
                           break; 
                                   }
                             }
                    }   
else {                                                                   // if Jack Instrument INPUT is detected -> act as STOP/FADE/TREMOLO
        
  while (digitalRead(Fade) == LOW ){ delay(100);   
                                     pressLength_milliSeconds = pressLength_milliSeconds + 100;  
                                   }


//Option 2 --> activate  TREMOLO -> hold switch for longer
     
    if (pressLength_milliSeconds >= optionTwo_milliSeconds){ 

          if(latchSim == 0)       { testStringsPos = 16;      // TREMOLO is ON -> DISPLAY "t"
                                    display7();
                              
                              
                              for (int pot = 0; pot > -1 ; pot = pot + inc) {
                                                          
                           //     intensity = analogRead(potPinFade)/11;       // 93 - 0
                            //    volume = (25500 - (pot * intensity))/100;    // 255 - 0

                           //   DigitalPotWrite(BOTH_POT_SEL , volume);
                                DigitalPotWrite(BOTH_POT_SEL , pot);
                              
                            //      if(pot >= 255){ inc = -5; }   // increment negative -> volume down
                             //     if(pot <= 0){ inc = 5; }      // increment positive -> volume up
                                    if(pot >= 255){ inc = -1; }   // increment negative -> volume down
                                    if(pot <= 0){ inc = 1; }  
                                    
                           //     speedTremolo = analogRead(potPinTremolo);
                            //    delay((1024-speedTremolo)/100);
                                  speedTremolo = (1124 - analogRead(potPinTremolo))/103;
                                  delay(speedTremolo);
                                  
                             stopState = digitalRead(Stop);
                             //delay(1);
                             if(stopState == LOW) { stopCount = 0; Count = 3; goto label;}

                             fadeState = digitalRead(Fade);
                             if(fadeState == LOW) { 
                             DigitalPotWrite(BOTH_POT_SEL , 255);
                             latchSim = 1;
                             pot = -2;}   

                              }

                     }
  
           else               { testStringsPos = 19;              // TREMOLO is OFF -> DISPLAY "."
                                display7();
                                latchSim = 0; 
                                pressLength_milliSeconds = 0; }
                                                                   }


//Option 1 -> Execute the first option if the button is held for less than half second -> simulate latching footswitch
//         -> FADE + STOP

        else if(pressLength_milliSeconds >= optionOne_milliSeconds){
                                  
                                  Count++;           
                                  testStringsPos = 11;
                                  display7();
                                  delay(75);
               label:             
           
                    switch (Count) {
                           //case 0:       // starting position
                           //potValue = 255; 
                           //testStringsPos = 12;
                           //display7();
                           //DigitalPotWrite(BOTH_POT_SEL , potValue);
                           //break;
                       
                              case 1:  // change the resistance on BOTH_POT from potValue to min:
                              for (int pot = potValue; pot >= 0; pot--) {
                              DigitalPotWrite(BOTH_POT_SEL , pot);
                 
                              delay(timeFade);

                              stopState = digitalRead(Stop);
                              delay(10);
                              if(stopState == LOW) { stopCount = 0; Count = 3; goto label;}

                              fadeState = digitalRead(Fade);
                              delay(10);
                              if(fadeState == LOW) { Count = 2; inter = pot; goto label;}    
                               
                                testStringsPos = pot/25;
                                display7();
                                                                        }
                                stopCount = 0;                                                   
                                inter = 0;
                                potValue = 0;
                              
                               break;  
                                     case 2:   // change the resistance on BOTH_POT from 0 up to potValue:
                                     potValue = 255;
                                   
                                     for (int pot = inter; pot <= potValue; pot++) {
                                     DigitalPotWrite(BOTH_POT_SEL , pot);
                                     
                                     delay(timeFadeUp);
                                     
                                     stopState = digitalRead(Stop);
                                     if(stopState == LOW) { stopCount = 0; Count = 3; goto label;}
                                       
                                     testStringsPos = pot/23;
                                     display7();
                                     
                                                                                             }
                                     stopCount = 0;                                                        
                                     Count = 0;
                                     break;

                                         case 3: // stop pressed
                                         // potValue = 0;   if you want to mute while STOP
                                         potValue = 255;
                                         DigitalPotWrite(BOTH_POT_SEL , potValue);
                                         testStringsPos = 15;
                                         display7();
                                            while(digitalRead(Stop) == LOW){digitalWrite(stopTip, LOW); 
                                            testStringsPos = 15;display7(); delay(250);
                                            digitalWrite(stopTip, HIGH); }
                                         stopCount = 1;                                                   
                                         inter = 0;
                                         delay(5);
                                         Count = 0;
                                       //  pressLength_milliSeconds = 0; 
                                         break;
    
                                    }
                                    
  //  pressLength_milliSeconds = 0; 
           
  
                                   
    pressLength_milliSeconds = 0; 
    
    }


// STOP option is always ready to go!

       
       stopState = digitalRead(Stop);
           if(stopState == LOW) { stopCount++;

             while(digitalRead(Stop) == LOW){digitalWrite(stopTip, LOW); 
             testStringsPos = 15;display7(); delay(250);
             digitalWrite(stopTip, HIGH); }
                               
                    switch (stopCount) {
                           case 1:       // stop
                           // potValue = 0;   if you want to mute while STOP
                           potValue = 255;
                           inter = 0; Count = 0; testStringsPos = 15; display7(); 
                           DigitalPotWrite(BOTH_POT_SEL , potValue);
                           delay(300);
                           break;

                           case 2:       // start -> go in FADE mode
                           potValue = 255; Count = 0; testStringsPos = 11; display7(); 
                           DigitalPotWrite(BOTH_POT_SEL , potValue);
                           stopCount = 0;
                           pressLength_milliSeconds = 0;
                           break;


                                       } 
 
                           } 
   }
}

/// END ///
